<?php
/**
 * Plugin generic functions file 
 *
 * @package Album and Image Gallery Plus Lightbox Pro
 * @since 1.0.0
 */

if ( !defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

/**
 * Update default settings
 * 
 * @package Album and Image Gallery Plus Lightbox Pro
 * @since 1.0.0
 */
function aigpl_pro_default_settings() {

	global $aigpl_pro_options;

	$aigpl_pro_options = array(
							'default_img'	=> '',
							'custom_css'	=> '',
						);

	$default_options = apply_filters( 'aigpl_options_default_values', $aigpl_pro_options );

	// Update default options
	update_option( 'aigpl_pro_options', $default_options );

	// Overwrite global variable when option is update
	$aigpl_pro_options = aigpl_pro_get_settings();
}

/**
 * Get Settings From Option Page
 * 
 * Handles to return all settings value
 * 
 * @package Album and Image Gallery Plus Lightbox Pro
 * @since 1.0.0
 */
function aigpl_pro_get_settings() {

	$options 	= get_option('aigpl_pro_options');
	$settings 	= is_array($options)  ? $options : array();

	return $settings;
}

/**
 * Get an option
 * Looks to see if the specified setting exists, returns default if not
 * 
 * @package Album and Image Gallery Plus Lightbox Pro
 * @since 1.0.0
 */
function aigpl_pro_get_option( $key = '', $default = false ) {
	global $aigpl_pro_options;

	$value = ! empty( $aigpl_pro_options[ $key ] ) ? $aigpl_pro_options[ $key ] : $default;
	$value = apply_filters( 'aigpl_get_option', $value, $key, $default );

	return apply_filters( 'aigpl_get_option_' . $key, $value, $key, $default );
}

/**
 * Escape Tags & Slashes
 *
 * Handles escapping the slashes and tags
 *
 * @package Album and Image Gallery Plus Lightbox Pro
 * @since 1.0.0
 */
function aigpl_pro_esc_attr($data) {
	return esc_attr( stripslashes($data) );
}

/**
 * Sanitize URL
 * 
 * @package Album and Image Gallery Plus Lightbox Pro
 * @since 1.0
 */
function aigpl_pro_clean_url( $url ) {
	return esc_url_raw( trim($url) );
}

/**
 * Clean variables using sanitize_text_field. Arrays are cleaned recursively.
 * Non-scalar values are ignored.
 * 
 * @package Album and Image Gallery Plus Lightbox Pro
 * @since 1.0
 */
function aigpl_pro_clean( $var ) {
	if ( is_array( $var ) ) {
		return array_map( 'aigpl_pro_clean', $var );
	} else {
		$data = is_scalar( $var ) ? sanitize_text_field( $var ) : $var;
		return wp_unslash($data);
	}
}

/**
 * Sanitize number value and return fallback value if it is blank
 * 
 * @package Album and Image Gallery Plus Lightbox Pro
 * @since 1.0
 */
function aigpl_pro_clean_number( $var, $fallback = null, $type = 'int' ) {

	if ( $type == 'number' ) {
		$data = intval( $var );
	} else {
		$data = absint( $var );
	}

	return ( empty($data) && isset($fallback) ) ? $fallback : $data;
}

/**
 * Sanitize Multiple HTML class
 * 
 * @package Album and Image Gallery Plus Lightbox Pro
 * @since 1.2.6
 */
function aigpl_pro_sanitize_html_classes($classes, $sep = " ") {
	$return = "";

	if( $classes && !is_array($classes) ) {
		$classes = explode($sep, $classes);
	}

	if( ! empty( $classes ) ) {
		foreach($classes as $class){
			$return .= sanitize_html_class($class) . " ";
		}
		$return = trim( $return );
	}

	return $return;
}

/**
 * Function to unique number value
 * 
 * @package Album and Image Gallery Plus Lightbox Pro
 * @since 1.0.0
 */
function aigpl_pro_get_unique() {
	static $unique = 0;
	$unique++;

	// For Elementor & Beaver Builder
	if( ( defined('ELEMENTOR_PLUGIN_BASE') && isset( $_POST['action'] ) && $_POST['action'] == 'elementor_ajax' )
	|| ( class_exists('FLBuilderModel') && ! empty( $_POST['fl_builder_data']['action'] ) ) ) {
		$unique = current_time('timestamp') . '-' . rand();
	}

	return $unique;
}

/**
 * Function to unique number value
 * 
 * @package Album and Image Gallery Plus Lightbox Pro
 * @since 1.0.0
 */
function aigpl_pro_unique_num() {
	static $unique = 0;
	$unique++;

	// For Elementor & Beaver Builder
	if( ( defined('ELEMENTOR_PLUGIN_BASE') && isset( $_POST['action'] ) && $_POST['action'] == 'elementor_ajax' )
	|| ( class_exists('FLBuilderModel') && ! empty( $_POST['fl_builder_data']['action'] ) ) ) {
		$unique = current_time('timestamp') . '-' . rand();
	}

	return $unique;
}

/**
 * Function to add array after specific key
 * 
 * @package Album and Image Gallery Plus Lightbox Pro
 * @since 1.0.0
 */
function aigpl_pro_add_array(&$array, $value, $index, $from_last = false) {

	if( is_array($array) && is_array($value) ) {

		if( $from_last ) {
			$total_count	= count($array);
			$index			= ( ! empty( $total_count ) && ( $total_count > $index ) ) ? ( $total_count - $index ): $index;
		}
		
		$split_arr 	= array_splice($array, max(0, $index));
		$array 		= array_merge( $array, $value, $split_arr);
	}

	return $array;
}

/**
 * Function to get post featured image
 * 
 * @package Album and Image Gallery Plus Lightbox Pro
 * @since 1.0.0
 */
function aigpl_pro_get_image_src( $post_id = '', $size = 'full', $default_img = false ) {
	$size 	= !empty($size) ? $size : 'full';
	$image 	= wp_get_attachment_image_src( $post_id, $size );

	if( !empty($image) ) {
		$image = isset($image[0]) ? $image[0] : '';
	}

	// Getting default image
	if( $default_img && empty($image) ) {
		$image = aigpl_pro_get_option( 'default_img' );
	}

	return $image;
}

/**
 * Function to get post excerpt
 * 
 * @package Album and Image Gallery Plus Lightbox Pro
 * @since 1.0.0
 */
function aigpl_pro_get_post_excerpt( $post_id = null, $content = '', $word_length = '55', $more = '...' ) {

	$word_length = !empty( $word_length ) ? $word_length : '55';

	// If post id is passed
	if( !empty($post_id) ) {
		if ( has_excerpt( $post_id ) ) {
			$content = get_the_excerpt();
		} else {
			$content = !empty($content) ? $content : get_the_content();
		}
	}

	if( !empty( $content ) ) {
		$content = strip_shortcodes( $content ); // Strip shortcodes
		$content = wp_trim_words( $content, $word_length, $more );
	}

	return $content;
}

/**
 * Pagination function for grid
 * 
 * @package WP Blog and Widgets Pro
 * @since 1.2.2
 */
function aigpl_pro_pagination($args = array()) {

	$big				= 999999999; // need an unlikely integer
	$page_links_temp	= array();	
	$pagination_type	= isset( $args['pagination_type'] ) ? $args['pagination_type'] : 'numeric';
	$multi_page			= ! empty( $args['multi_page'] ) 	? 1 : 0;

	$paging = array(
		'base' 		=> str_replace( $big, '%#%', esc_url( get_pagenum_link( $big ) ) ),
		'format' 	=> '?paged=%#%',
		'current' 	=> max( 1, $args['paged'] ),
		'total' 	=> $args['total'],
		'prev_next'	=> true,
		'prev_text'	=> '&laquo; '.__('Previous', 'album-and-image-gallery-plus-lightbox'),
		'next_text'	=> __('Next', 'album-and-image-gallery-plus-lightbox').' &raquo;',
	);

	// If pagination is prev-next and shortcode is placed in single post
	if( $multi_page ) {
		$paging['type']		= ( $pagination_type == 'prev-next' ) ? 'array' : 'plain';
		$paging['base']		= esc_url_raw( add_query_arg( 'album_page', '%#%', false ) );
		$paging['format']	= '?album_page=%#%';
	}

	$page_links = paginate_links( apply_filters( 'wpbaw_pro_paging_args', $paging ) );

	// For single post shortcode we just fetch the prev-next link
	if( $pagination_type == 'prev-next' && $page_links && is_array( $page_links ) ) {

		foreach ($page_links as $page_link_key => $page_link) {
			if( strpos( $page_link, 'next page-numbers') !== false || strpos( $page_link, 'prev page-numbers') !== false ) {
				$page_links_temp[ $page_link_key ] = $page_link;
			}
		}
		return join( "\n", $page_links_temp );
	}

	return $page_links;
}

/**
 * Function to get album gallery designs
 * 
 * @package Album and Image Gallery Plus Lightbox Pro
 * @since 1.0.0
 */
function aigpl_pro_gallery_designs() {
	$design_arr = array(
					'design-1'  => __('Gallery Design 1', 'album-and-image-gallery-plus-lightbox'),
					'design-2'  => __('Gallery Design 2', 'album-and-image-gallery-plus-lightbox'),
					'design-3'  => __('Gallery Design 3', 'album-and-image-gallery-plus-lightbox'),
					'design-4'  => __('Gallery Design 4', 'album-and-image-gallery-plus-lightbox'),
					'design-5'  => __('Gallery Design 5', 'album-and-image-gallery-plus-lightbox'),
					'design-6'  => __('Gallery Design 6', 'album-and-image-gallery-plus-lightbox'),
					'design-7'  => __('Gallery Design 7', 'album-and-image-gallery-plus-lightbox'),
					'design-8'  => __('Gallery Design 8', 'album-and-image-gallery-plus-lightbox'),
					'design-9'  => __('Gallery Design 9', 'album-and-image-gallery-plus-lightbox'),
					'design-10' => __('Gallery Design 10', 'album-and-image-gallery-plus-lightbox'),
					'design-11' => __('Gallery Design 11', 'album-and-image-gallery-plus-lightbox'),
					'design-12' => __('Gallery Design 12', 'album-and-image-gallery-plus-lightbox'),
					'design-13' => __('Gallery Design 13', 'album-and-image-gallery-plus-lightbox'),
					'design-14' => __('Gallery Design 14', 'album-and-image-gallery-plus-lightbox'),
					'design-15' => __('Gallery Design 15', 'album-and-image-gallery-plus-lightbox'),
				);
	return apply_filters( 'aigpl_pro_gallery_designs', $design_arr );
}

/**
 * Function to get album designs
 * 
 * @package Album and Image Gallery Plus Lightbox Pro
 * @since 1.0.0
 */
function aigpl_pro_album_designs() {
	$design_arr = array(
				'design-1'  => __('Album Design 1', 'album-and-image-gallery-plus-lightbox'),
				'design-2'  => __('Album Design 2', 'album-and-image-gallery-plus-lightbox'),
				'design-3'  => __('Album Design 3', 'album-and-image-gallery-plus-lightbox'),
				'design-4'  => __('Album Design 4', 'album-and-image-gallery-plus-lightbox'),
				'design-5'  => __('Album Design 5', 'album-and-image-gallery-plus-lightbox'),
				'design-6'  => __('Album Design 6', 'album-and-image-gallery-plus-lightbox'),
				'design-7'  => __('Album Design 7', 'album-and-image-gallery-plus-lightbox'),
				'design-8'  => __('Album Design 8', 'album-and-image-gallery-plus-lightbox'),
				'design-9'  => __('Album Design 9', 'album-and-image-gallery-plus-lightbox'),
				'design-10' => __('Album Design 10', 'album-and-image-gallery-plus-lightbox'),
				'design-11' => __('Album Design 11', 'album-and-image-gallery-plus-lightbox'),
				'design-12' => __('Album Design 12', 'album-and-image-gallery-plus-lightbox'),
				'design-13' => __('Album Design 13', 'album-and-image-gallery-plus-lightbox'),
				'design-14' => __('Album Design 14', 'album-and-image-gallery-plus-lightbox'),
				'design-15' => __('Album Design 15', 'album-and-image-gallery-plus-lightbox'),
				'design-16' => __('Album Design 16', 'album-and-image-gallery-plus-lightbox'),
				'design-17' => __('Album Design 17', 'album-and-image-gallery-plus-lightbox'),
			);
	return apply_filters( 'aigpl_pro_album_designs', $design_arr );
}

/**
 * Function to get shortocdes registered in plugin
 * 
 * @package Album and Image Gallery Plus Lightbox Pro
 * @since 1.2.6
 */
function aigpl_pro_registered_shortcodes() {
	$shortcodes = array(
					'aigpl-gallery-album'			=> __('Album Grid', 'album-and-image-gallery-plus-lightbox'),
					'aigpl-gallery-album-slider'	=> __('Album Slider', 'album-and-image-gallery-plus-lightbox'),
					'aigpl-gallery'					=> __('Gallery Grid', 'album-and-image-gallery-plus-lightbox'),
					'aigpl-gallery-slider'			=> __('Gallery Slider', 'album-and-image-gallery-plus-lightbox'),
				);
	return apply_filters( 'aigpl_pro_registered_shortcodes', (array)$shortcodes );
}